package ca.canadapost.cpcdp.returns;

import ca.canadapost.cpcdp.returns.generated.messages.Messages;
import ca.canadapost.cpcdp.returns.generated.openreturn.*;

import com.sun.jersey.api.client.*;
import com.sun.jersey.api.client.config.*;

import javax.xml.bind.JAXBContext;

import java.io.FileInputStream;
import java.io.InputStream;
import java.util.Iterator;
import java.util.Properties;

import java.math.*;

/**
 * Sample code for the CreateOpenReturnTemplate Canada Post service.
 * <p>
 * The CreateOpenReturnTemplate service is used to request creation of generic
 * labels for retrieval and printing. The sender address and parcel weight are
 * unknown. Each label has a unique barcode, but the rest of the data is the same.
 * The labels can be distributed as part of the original shipment, or sent to a
 * specific individual. 
 * <p>
 * This sample is configured to access the Developer Program sandbox environment. 
 * Use your development key username and password for the web service credentials.
 * <p>
 * Jersey REST client using JAXB for CreateOpenReturn Request.  <br>
 *  USAGE:<pre>
 *        CreateOpenReturnTemplateClient client = new CreateOpenReturnTemplateClient(username, password);
 *        ClientResponse response = client.createOpenReturnTemplate(return);
 *        // do whatever with response
 *        client.close();
 *  </pre>
 *
 */
public class CreateOpenReturnTemplateClient {

    private Client aClient;
    private static String LINK;
    
    // Main class for local testing only
    public static void main(String[] args) {
    	
    	// Your username, password and customer number are imported from the following file
    	// CPCWS_Returns_Java_Samples/user.properties 
    	Properties userProps = new Properties();
    	FileInputStream propInputStream;
		try {
			propInputStream = new FileInputStream("user.properties");
			userProps.load(propInputStream);
			propInputStream.close(); // better in finally block
		} catch (Exception e) {
			// TODO Auto-generated catch block
			e.printStackTrace(System.out);
			return;
		}
    	String username = userProps.getProperty("username");
    	String password = userProps.getProperty("password");
    	String mailedBy = userProps.getProperty("mailedBy");    	
    	String mobo = mailedBy;    	
    	
    	// REST URL
    	LINK = "https://ct.soa-gw.canadapost.ca/rs/" + mailedBy + "/" + mobo + "/openreturn";
    	
		// Create CreateOpenReturn XML Request Object
        OpenReturn myReturn = new OpenReturn();
              
		myReturn.setMaxNumberOfArtifacts(new BigInteger("10"));
						
		DomesticAddressDetailsType receiverAddress = new DomesticAddressDetailsType();
		receiverAddress.setAddressLine1("123 Postal Drive");
		receiverAddress.setCity("Ottawa");
		receiverAddress.setProvince("ON");
		receiverAddress.setPostalCode("K1P5Z9");
		
		DomesticAddressType receiver = new DomesticAddressType();
		receiver.setName("John Doe");
		receiver.setCompany("ACME Corp.");
		receiver.setDomesticAddress(receiverAddress);
				
		PrintPreferencesType printPreferences = new PrintPreferencesType();
		printPreferences.setEncoding("PDF");
		printPreferences.setOutputFormat("8.5x11");
		
		SettlementInfoType settlementInfo = new SettlementInfoType();
		settlementInfo.setContractId("0042708517");
						
		myReturn.setServiceCode("DOM.EP");
		myReturn.setReceiver(receiver);
		myReturn.setPrintPreferences(printPreferences);
		myReturn.setSettlementInfo(settlementInfo);
    	
		// Execute CreateOpenReturn Request
        CreateOpenReturnTemplateClient myClient = new CreateOpenReturnTemplateClient(username, password);
        ClientResponse resp = myClient.createOpenReturnTemplate(myReturn);
        InputStream respIS = resp.getEntityInputStream();
        
        System.out.println("HTTP Response Status: " + resp.getStatus() + " " + resp.getClientResponseStatus());

        // Example of using JAXB to parse xml response
        JAXBContext jc;
        try {
        	jc = JAXBContext.newInstance(OpenReturnInfo.class, Messages.class);
            Object entity = jc.createUnmarshaller().unmarshal(respIS);
            // Determine whether response data matches CreateOpenReturnInfo schema.
            if (entity instanceof OpenReturnInfo) {
            	OpenReturnInfo returnInfo = (OpenReturnInfo) entity;
            	System.out.println("Artifacts Remaining: " + returnInfo.getArtifactsRemaining());                
                for (Iterator<Link> iter = returnInfo.getLinks().getLinks().iterator(); iter.hasNext();) { 
                	Link aItem = (Link) iter.next();                	
	                System.out.println( aItem.getRel() + ": " + aItem.getHref());
                }
            } else {
                // Assume Error Schema
                Messages messageData = (Messages) entity;
                for (Iterator<Messages.Message> iter = messageData.getMessage().iterator(); iter.hasNext();) {
                    Messages.Message aMessage = (Messages.Message) iter.next();
                    System.out.println("Error Code: " + aMessage.getCode());
                    System.out.println("Error Msg: " + aMessage.getDescription());
                }
            }
        } catch (Exception e) {
        	e.printStackTrace(System.out);
        }

        myClient.close();
    }

    public CreateOpenReturnTemplateClient(String username, String password) {
        ClientConfig config = new DefaultClientConfig();
        aClient = Client.create(config);
        aClient.addFilter(new com.sun.jersey.api.client.filter.HTTPBasicAuthFilter(username, password));
    }

    public ClientResponse createOpenReturnTemplate(Object xml) throws UniformInterfaceException {
        WebResource aWebResource = aClient.resource(LINK);
        return aWebResource.accept("application/vnd.cpc.openreturn-v2+xml").header("Content-Type", "application/vnd.cpc.openreturn-v2+xml").acceptLanguage("en-CA").post(ClientResponse.class, xml);
    }

    public void close() {
        aClient.destroy();
    }
}
